"""General interfaces for assembly-like languages."""
from abc import ABC, abstractmethod
from typing import List

from symbolic_execution.program.block.interfaces import IInstructionBlock


class ILanguage(ABC):
    """Abstract class to represent an assembly-like language."""

    @abstractmethod
    def tokenize_code(self, code: str) -> List[str]:
        """Parse {code} & returns the tokens."""
        pass

    @abstractmethod
    def build_control_flow_graph(self, tokens: List[str]) -> IInstructionBlock:
        """Parse the tokens and return the control flow graph."""
        pass
