"""Interfaces for executers.

An executer is in charge to run a block of instructions on a machine.
"""
from abc import ABC, abstractmethod
from symbolic_execution.hardware.interfaces import IMachine
from ..block.interfaces import IInstructionBlock


class IExecuter(ABC):
    """An executer is in charge to run a block of instructions on a machine.

    A same block of instructions can be executed by different executers to support
    different use cases. Example, in order to support both concrete and symbolic
    execution.
    """

    @abstractmethod
    def execute(self, block: IInstructionBlock, machine: IMachine) -> IMachine:
        """Execute the block of instruction over the machine."""
        pass
