"""General interface for an instruction."""
from abc import ABC, abstractmethod

from symbolic_execution.hardware.interfaces import IMachine


class IInstruction(ABC):
    """Abstract class to represent an instruction."""

    @property
    def pc(self) -> int:
        """Opcode representing the instruction in a language.

        It takes the pc of the instruction within the program to initialize.
        """
        return self._pc

    def __init__(self, pc: int, **args):
        """Initialize an instruction for a language.

        In particular, the instruction is associated with an byte.
        """
        self._pc = pc

    @abstractmethod
    def run(self, machine: IMachine):
        """Run the instruction on a machine."""
        machine.pc = machine.pc + 1

    def __repr__(self) -> str:
        """Return the string representation of the instruction."""
        return f"{self.pc}: {self.__class__.__name__}"


class IConditionalBranchingOperator(IInstruction):
    """Every conditional branching operator should inherit from that class."""


class IUnconditionalBranchingOperator(IInstruction):
    """Every conditional branching operator should inherit from that class."""
